unit uClientWin32;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, Mask,
  XmlRpcClient, XmlRpcTypes;

type
  TFrmClienteWin32 = class(TForm)
    Panel1: TPanel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    Panel2: TPanel;
    btnFechar: TBitBtn;
    Label1: TLabel;
    edtNome: TMaskEdit;
    btnFormatar: TBitBtn;
    edtResultado: TMaskEdit;
    Label2: TLabel;
    TabSheet5: TTabSheet;
    Label3: TLabel;
    Label4: TLabel;
    edtHost: TMaskEdit;
    edtPort: TMaskEdit;
    edtCasasDecimais: TEdit;
    Label5: TLabel;
    btnCalcPI: TBitBtn;
    mPI: TMemo;
    Label6: TLabel;
    edtLogin: TMaskEdit;
    edtSenha: TMaskEdit;
    Label7: TLabel;
    Label8: TLabel;
    mStatus: TMemo;
    btnPostar: TBitBtn;
    Label9: TLabel;
    edtMes: TMaskEdit;
    Label10: TLabel;
    edtAno: TMaskEdit;
    btnCalcFases: TBitBtn;
    lstbxFases: TListBox;
    Label11: TLabel;
    Memo1: TMemo;
    Label12: TLabel;
    procedure btnFormatarClick(Sender: TObject);
    procedure btnCalcPIClick(Sender: TObject);
    procedure btnFecharClick(Sender: TObject);
    procedure btnPostarClick(Sender: TObject);
    procedure btnCalcFasesClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }

    function _FormatarData(dd, mm, yyyy: integer): String;

    function CreateRpcCaller: TRpcCaller;
    function CreateRpcFunction(prstFunction: String): IRPCFunction;
    procedure FormatarNome;
    procedure CalcularPI;
    procedure CalcularFasesLua;
    procedure PostarTwitter;

  public
    { Public declarations }
  end;

var
  FrmClienteWin32: TFrmClienteWin32;

implementation

{$R *.dfm}

procedure TFrmClienteWin32.btnFormatarClick(Sender: TObject);
begin
  FormatarNome;
end;

procedure TFrmClienteWin32.FormatarNome;
var
  rpcCaller: TRpcCaller;
  rpcFunction: IRpcFunction;
  rpcResult: IRpcResult;
begin
  rpcCaller := CreateRpcCaller;
  try
    rpcFunction := CreateRpcFunction('app.formatarNome');
    rpcFunction.AddItem(edtNome.Text);
    rpcResult := rpcCaller.Execute(rpcFunction);

    if rpcResult.IsError then
      MessageDlg(rpcResult.ErrorMsg, mtError, [mbOK], 0)
    else
      edtResultado.Text := rpcResult.AsString;
  finally
    rpcCaller.Free;
  end;
end;

function TFrmClienteWin32.CreateRpcCaller: TRpcCaller;
begin
  Result := TRpcCaller.Create;
  Result.EndPoint := '';
  Result.HostName := edtHost.Text;
  Result.HostPort := StrToInt(edtPort.Text);
end;

function TFrmClienteWin32.CreateRpcFunction(
  prstFunction: String): IRPCFunction;
begin
  Result := TRpcFunction.Create;
  Result.ObjectMethod := prstFunction;
end;

procedure TFrmClienteWin32.btnCalcPIClick(Sender: TObject);
begin
  try
    Screen.Cursor := crHourGlass;
    CalcularPI;
  finally
    Screen.Cursor:= crDefault;
  end;
end;

procedure TFrmClienteWin32.btnFecharClick(Sender: TObject);
begin
  Close;
end;

procedure TFrmClienteWin32.btnPostarClick(Sender: TObject);
begin
  try
    Screen.Cursor := crHourGlass;
    PostarTwitter;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFrmClienteWin32.btnCalcFasesClick(Sender: TObject);
begin
  CalcularFasesLua;
end;

procedure TFrmClienteWin32.CalcularFasesLua;
var
  rpcCaller: TRpcCaller;
  rpcFunction: IRpcFunction;
  rpcResult: IRpcResult;
  lDia: integer;
  lMes: integer;
  lAno: integer;
begin
  lMes := StrToInt(edtMes.Text);
  lAno := StrToInt(edtAno.Text);

  rpcCaller := CreateRpcCaller;

  try
    rpcFunction := CreateRpcFunction('app.obterFasesLua');
    rpcFunction.AddItem(lMes);
    rpcFunction.AddItem(lAno);
    rpcResult := rpcCaller.Execute(rpcFunction);

    if rpcResult.IsError then
      MessageDlg(rpcResult.ErrorMsg, mtError, [mbOK], 0)
    else
    begin
      lstbxFases.Items.Clear;

      for lDia := 1 to rpcResult.AsArray.Count do
        lstbxFases.Items.Add(_FormatarData(lDia, lMes, lAno) + '= ' +
        rpcResult.AsArray.Items[lDia - 1].AsString);
    end;
  finally
    rpcCaller.Free;
  end;
end;

procedure TFrmClienteWin32.CalcularPI;
var
  rpcCaller: TRpcCaller;
  rpcFunction: IRpcFunction;
  rpcResult: IRpcResult;
begin
  rpcCaller := CreateRpcCaller;

  try
    rpcFunction := CreateRpcFunction('app.obterPI');
    rpcFunction.AddItem(StrToInt(edtCasasDecimais.Text));
    rpcResult := rpcCaller.Execute(rpcFunction);

    if rpcResult.IsError then
      MessageDlg(rpcResult.ErrorMsg, mtError, [mbOK], 0)
    else
      mPI.Lines.Text := rpcResult.AsString;
  finally
    rpcCaller.Free;
  end;
end;

procedure TFrmClienteWin32.PostarTwitter;
var
  rpcCaller: TRpcCaller;
  rpcFunction: IRpcFunction;
  rpcResult: IRpcResult;
begin
  rpcCaller := CreateRpcCaller;
  
  try
    rpcFunction := CreateRpcFunction('app.postarTwitter');
    rpcFunction.AddItem(edtLogin.Text);
    rpcFunction.AddItem(edtSenha.Text);
    rpcFunction.AddItem(mStatus.Text);
    rpcResult := rpcCaller.Execute(rpcFunction);

    if rpcResult.IsError then
      MessageDlg(rpcResult.ErrorMsg, mtError, [mbOK], 0)
    else
      MessageDlg(rpcResult.AsString, mtInformation, [mbOK], 0);
   finally
     rpcCaller.Free;
   end;
end;

function TFrmClienteWin32._FormatarData(dd, mm, yyyy: integer): String;
var
  lstDD: String;
  lstMM: String;
  lstYYYY: String;
begin
  lstDD := IntToStr(dd);
  lstMM := IntToStr(mm);
  lstYYYY := IntToStr(yyyy);

  if Length(lstDD) = 1 then
    lstDD := '0' + lstDD;

  if Length(lstMM) = 1 then
    lstMM := '0' + lstMM;

  Result := lstDD + '/' + lstMM + '/' + lstYYYY;
end;

procedure TFrmClienteWin32.FormActivate(Sender: TObject);
var
  lMes: Word;
  lAno: Word;
  lDia: Word;  // ignora
begin
  DecodeDate(Date, lAno, lMes, lDia);
  edtMes.Text := IntToStr(lMes);
  edtAno.Text := IntToStr(lAno);
end;

end.
