unit ufrmPrincipal;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DBXpress, FMTBcd, DB, SqlExpr, IBDatabase;

type
  TfrmPrincipal = class(TForm)
    Label1: TLabel;
    lblTempoModoPadrao: TLabel;
    Label2: TLabel;
    lblTempoModoSelect: TLabel;
    btnGerarTxtImportacao: TButton;
    btnImportarModoPadrao: TButton;
    btnImportarModoSelect: TButton;
    SqlConn: TSQLConnection;
    qryImportacaoPadrao: TSQLQuery;
    qryAuxCriarBanco: TSQLQuery;
    qryImportacaoSelect: TSQLQuery;
    procedure btnGerarTxtImportacaoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnImportarModoPadraoClick(Sender: TObject);
    procedure btnImportarModoSelectClick(Sender: TObject);
  private
    FPathBanco: string;
    procedure GerarTxtImportacao;
    procedure CriarBanco;
    procedure ImportarModoPadrao;
    procedure ImportarModoSelect;
  public
    { Public declarations }
  end;

var
  frmPrincipal: TfrmPrincipal;

implementation

{$R *.dfm}

type
  TStrArray = array of string;

// from: http://www.swissdelphicenter.ch/torry/showcode.php?id=1326
function Explode(var a: TStrArray; Border, S: string): Integer;
var
  S2: string;
begin
  Result  := 0;
  S2 := S + Border;
  repeat
    SetLength(A, Length(A) + 1);
    a[Result] := Copy(S2, 0,Pos(Border, S2) - 1);
    Delete(S2, 1,Length(a[Result] + Border));
    Inc(Result);
  until S2 = '';
end;

procedure TfrmPrincipal.GerarTxtImportacao;
var
  i: Integer;
begin
  with TStringList.Create do
  try
    for i := 1 to 5000 do
      Add('Nome' + IntToStr(i) + ';' + 'endereco' + IntToStr(i) + ';' + 'telefone' + IntToStr(i));

    SaveToFile('clientes.txt');
  finally
    Free;
  end;
end;

procedure TfrmPrincipal.CriarBanco;
begin
  SqlConn.Connected := False;

  if FileExists(FPathBanco) then
    DeleteFile(FPathBanco);

  with TIBDatabase.Create(nil) do
  try
    DatabaseName := FPathBanco;
    SQLDialect   := 3;
    Params.Clear;
    Params.Add('USER ''SYSDBA''');
    Params.Add('PASSWORD ''masterkey''');
    Params.Add('PAGE_SIZE 16384');
    Params.Add('DEFAULT CHARACTER SET ISO8859_1');
    CreateDatabase;
  finally
    Free;
  end;

  with qryAuxCriarBanco, Sql do
  begin
    Clear;
    Add('CREATE TABLE CLIENTE (');
    Add('  ID INTEGER NOT NULL PRIMARY KEY,');
    Add('  NOME VARCHAR(50),');
    Add('  ENDERECO  VARCHAR(50),');
    Add('  TELEFONE  VARCHAR(20)');
    Add(')');
    ExecSql;

    Clear;
    Add('CREATE GENERATOR GEN_CLIENTE_ID');
    ExecSql;
  end;
end;

procedure TfrmPrincipal.ImportarModoPadrao;
var
  F: TextFile;
  sLinha: string;
  HoraIni: TDateTime;
  campos: TStrArray;
begin
  lblTempoModoPadrao.Caption := '';

  CriarBanco;
  
  HoraIni := Now;
  try
    AssignFile(F, 'clientes.txt');
    try
      Reset(F);
      while not Eof(F) do
      begin
        ReadLn(F, sLinha);

        Explode(campos, ';', sLinha);

        with qryImportacaoPadrao do
        begin
          ParamByName('NOME').AsString := campos[0];
          ParamByName('ENDERECO').AsString := campos[1];
          ParamByName('TELEFONE').AsString := campos[2];
          ExecSql;
        end;

      end;
    finally
      CloseFile(F);
    end;
  finally
    lblTempoModoPadrao.Caption := FormatDateTime('hh:nn:ss', Now - HoraIni);
  end;

end;

procedure TfrmPrincipal.ImportarModoSelect;

  procedure ExecutarInsert(const selects: string);
  begin
    qryImportacaoSelect.Sql.Text := 'INSERT INTO CLIENTE ' + selects;
    qryImportacaoSelect.ExecSql;
  end;

var
  HoraIni: TDateTime;
  F: TextFile;
  sLinha, sSelects: string;
  campos: TStrArray;
  iQtdeUnions: Integer;
  
begin
  lblTempoModoSelect.Caption := '';

  CriarBanco;

  sSelects := '';

  HoraIni := Now;
  try
    AssignFile(F, 'clientes.txt');
    try
      Reset(F);

      iQtdeUnions := 0;
      
      while not Eof(F) do
      begin
        ReadLn(F, sLinha);

        Explode(campos, ';', sLinha);

        if sSelects <> '' then
        begin
          AppendStr(sSelects, ' UNION ');
          Inc(iQtdeUnions);
        end;
        
        AppendStr(sSelects, 'SELECT ');
        AppendStr(sSelects, 'GEN_ID(GEN_CLIENTE_ID,1), ');
        AppendStr(sSelects, 'CAST(' + QuotedStr(campos[0]) + ' AS VARCHAR(50)), ');
        AppendStr(sSelects, 'CAST(' + QuotedStr(campos[1]) + ' AS VARCHAR(50)), ');
        AppendStr(sSelects, 'CAST(' + QuotedStr(campos[2]) + ' AS VARCHAR(20)) ' );
        AppendStr(sSelects, 'FROM ');
        AppendStr(sSelects, 'RDB$DATABASE');

        if (iQtdeUnions > 0) and (iQtdeUnions mod 127 = 0) then
        begin
          ExecutarInsert(sSelects);
          sSelects := '';
          iQtdeUnions := 0;
        end;

      end;

      if sSelects <> '' then
        ExecutarInsert(sSelects);

    finally
      CloseFile(F);
    end;
  finally
    lblTempoModoSelect.Caption := FormatDateTime('hh:nn:ss', Now - HoraIni);
  end;

end;


procedure TfrmPrincipal.btnGerarTxtImportacaoClick(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    GerarTxtImportacao;
  finally
    Screen.Cursor := crDefault;
  end;
  
  ShowMessage('Arquivo gerado com sucesso');
end;



procedure TfrmPrincipal.FormCreate(Sender: TObject);
begin
  FPathBanco := IncludeTrailingPathDelimiter(ExtractFileDir(Application.ExeName)) + 'clientes.fdb';
  SqlConn.Params.Values['DATABASE'] := FPathBanco;
end;

procedure TfrmPrincipal.btnImportarModoPadraoClick(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    ImportarModoPadrao;
  finally
    Screen.Cursor := crDefault;
  end;

  ShowMessage('Importao concluda');
end;

procedure TfrmPrincipal.btnImportarModoSelectClick(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    ImportarModoSelect;
  finally
    Screen.Cursor := crDefault;
  end;

  ShowMessage('Importao concluda');
end;

end.
